@@ -1,4 +1,4 @@
-from toolchain_profiler import ToolchainProfiler
+from .toolchain_profiler import ToolchainProfiler
 import time, os, sys, logging
 from subprocess import Popen, PIPE, STDOUT
 
@@ -6,7 +6,7 @@ TRACK_PROCESS_SPAWNS = True if (os.getenv('EM_BUILD_VERBOSE') and int(os.getenv(
 WORKING_ENGINES = {} # Holds all configured engines and whether they work: maps path -> True/False
 __rootpath__ = os.path.dirname(os.path.dirname(__file__))
 
-def timeout_run(proc, timeout=None, note='unnamed process', full_output=False):
+def timeout_run(proc, timeout=None, note='unnamed process', full_output=False, note_args=[], throw_on_failure=True):
   start = time.time()
   if timeout is not None:
     while time.time() - start < timeout and proc.poll() is None:
@@ -15,7 +15,9 @@ def timeout_run(proc, timeout=None, note='unnamed process', full_output=False):
       proc.kill() # XXX bug: killing emscripten.py does not kill it's child process!
       raise Exception("Timed out: " + note)
   out = proc.communicate()
-  out = map(lambda o: '' if o is None else o, out)
+  out = ['' if o is None else o for o in out]
+  if throw_on_failure and proc.returncode != 0:
+    raise Exception('Subprocess "' + ' '.join(note_args) + '" failed with exit code ' + str(proc.returncode) + '!')
   if TRACK_PROCESS_SPAWNS:
     logging.info('Process ' + str(proc.pid) + ' finished after ' + str(time.time() - start) + ' seconds. Exit code: ' + str(proc.returncode))
   return '\n'.join(out) if full_output else out[0]
@@ -54,7 +56,7 @@ def check_engine(engine):
     logging.debug('Checking JS engine %s' % engine)
     if 'hello, world!' in run_js(os.path.join(__rootpath__, 'src', 'hello_world.js'), engine, skip_check=True):
       WORKING_ENGINES[engine_path] = True
-  except Exception, e:
+  except Exception as e:
     logging.info('Checking JS engine %s failed. Check your config file. Details: %s' % (str(engine), str(e)))
     WORKING_ENGINES[engine_path] = False
   return WORKING_ENGINES[engine_path]
@@ -98,8 +100,9 @@ def run_js(filename, engine=None, args=[], check_timeout=False, stdin=None, stdo
         stdin=stdin,
         stdout=stdout,
         stderr=stderr,
-        cwd=cwd)
-  except Exception, e:
+        cwd=cwd,
+        universal_newlines=True)
+  except Exception as e:
     # the failure may be because the engine is not present. show the proper
     # error in that case
     if not skip_check:
@@ -116,8 +119,9 @@ def run_js(filename, engine=None, args=[], check_timeout=False, stdin=None, stdo
       proc,
       timeout,
       'Execution',
-      full_output=full_output)
-  except Exception, e:
+      full_output=full_output,
+      throw_on_failure=False)
+  except Exception as e:
     # the failure may be because the engine does not work. show the proper
     # error in that case
     if not skip_check:
