@@ -5,9 +5,12 @@ WebIDL binder
 http://kripken.github.io/emscripten-site/docs/porting/connecting_cpp_and_javascript/WebIDL-Binder.html
 '''
 
+from __future__ import print_function
 import os, sys
 
-import shared
+sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from tools import shared
 
 sys.path.append(shared.path_from_root('third_party'))
 sys.path.append(shared.path_from_root('third_party', 'ply'))
@@ -23,11 +26,11 @@ CHECKS = os.environ.get('IDL_CHECKS') or 'DEFAULT'
 # DEBUG=1 will print debug info in render_function
 DEBUG = os.environ.get('IDL_VERBOSE') is '1'
 
-if DEBUG: print "Debug print ON, CHECKS=%s" % CHECKS
+if DEBUG: print("Debug print ON, CHECKS=%s" % CHECKS)
 
-class Dummy:
+class Dummy(object):
   def __init__(self, init):
-    for k, v in init.iteritems():
+    for k, v in init.items():
       self.__dict__[k] = v
 
   def getExtendedAttribute(self, name):
@@ -73,14 +76,14 @@ mid_c += [r'''
 extern "C" {
 ''']
 
-def emit_constructor(name):
-  global mid_js
-  mid_js += [r'''%s.prototype = %s;
-%s.prototype.constructor = %s;
-%s.prototype.__class__ = %s;
-%s.__cache__ = {};
-Module['%s'] = %s;
-''' % (name, 'Object.create(%s.prototype)' % (implements[name][0] if implements.get(name) else 'WrapperObject'), name, name, name, name, name, name, name)]
+def build_constructor(name):
+  implementing_name = implements[name][0] if implements.get(name) else 'WrapperObject'
+  return [r'''{name}.prototype = Object.create({implementing}.prototype);
+{name}.prototype.constructor = {name};
+{name}.prototype.__class__ = {name};
+{name}.__cache__ = {{}};
+Module['{name}'] = {name};
+'''.format(name=name, implementing=implementing_name)]
 
 
 mid_js += ['''
@@ -90,7 +93,7 @@ function WrapperObject() {
 }
 ''']
 
-emit_constructor('WrapperObject')
+mid_js += build_constructor('WrapperObject')
 
 mid_js += ['''
 function getCache(__class__) {
@@ -138,9 +141,9 @@ function getClass(obj) {
 }
 Module['getClass'] = getClass;
 
-// Converts a value into a C-style string, storing it in temporary space
+// Converts big (string or array) values into a C-style storage, in temporary space
 
-var ensureStringCache = {
+var ensureCache = {
   buffer: 0,  // the main buffer of temporary storage
   size: 0,   // the size of buffer
   pos: 0,    // the next free offset in buffer
@@ -148,49 +151,106 @@ var ensureStringCache = {
   needed: 0, // the total size we need next time
 
   prepare: function() {
-    if (this.needed) {
+    if (ensureCache.needed) {
       // clear the temps
-      for (var i = 0; i < this.temps.length; i++) {
-        Module['_free'](this.temps[i]);
+      for (var i = 0; i < ensureCache.temps.length; i++) {
+        Module['_free'](ensureCache.temps[i]);
       }
-      this.temps.length = 0;
+      ensureCache.temps.length = 0;
       // prepare to allocate a bigger buffer
-      Module['_free'](this.buffer);
-      this.buffer = 0;
-      this.size += this.needed;
+      Module['_free'](ensureCache.buffer);
+      ensureCache.buffer = 0;
+      ensureCache.size += ensureCache.needed;
       // clean up
-      this.needed = 0;
+      ensureCache.needed = 0;
     }
-    if (!this.buffer) { // happens first time, or when we need to grow
-      this.size += 100; // heuristic, avoid many small grow events
-      this.buffer = Module['_malloc'](this.size);
-      assert(this.buffer);
+    if (!ensureCache.buffer) { // happens first time, or when we need to grow
+      ensureCache.size += 128; // heuristic, avoid many small grow events
+      ensureCache.buffer = Module['_malloc'](ensureCache.size);
+      assert(ensureCache.buffer);
     }
-    this.pos = 0;
+    ensureCache.pos = 0;
   },
-  alloc: function(value) {
-    assert(this.buffer);
-    var array = intArrayFromString(value);
-    var len = array.length;
+  alloc: function(array, view) {
+    assert(ensureCache.buffer);
+    var bytes = view.BYTES_PER_ELEMENT;
+    var len = array.length * bytes;
+    len = (len + 7) & -8; // keep things aligned to 8 byte boundaries
     var ret;
-    if (this.pos + len >= this.size) {
-      // we failed to allocate in the buffer, this time around :(
+    if (ensureCache.pos + len >= ensureCache.size) {
+      // we failed to allocate in the buffer, ensureCache time around :(
       assert(len > 0); // null terminator, at least
-      this.needed += len;
+      ensureCache.needed += len;
       ret = Module['_malloc'](len);
-      this.temps.push(ret);
+      ensureCache.temps.push(ret);
     } else {
       // we can allocate in the buffer
-      ret = this.buffer + this.pos;
-      this.pos += len;
+      ret = ensureCache.buffer + ensureCache.pos;
+      ensureCache.pos += len;
     }
-    writeArrayToMemory(array, ret);
     return ret;
   },
+  copy: function(array, view, offset) {
+    var offsetShifted = offset;
+    var bytes = view.BYTES_PER_ELEMENT;
+    switch (bytes) {
+      case 2: offsetShifted >>= 1; break;
+      case 4: offsetShifted >>= 2; break;
+      case 8: offsetShifted >>= 3; break;
+    }
+    for (var i = 0; i < array.length; i++) {
+      view[offsetShifted + i] = array[i];
+    }
+  },
 };
 
 function ensureString(value) {
-  if (typeof value === 'string') return ensureStringCache.alloc(value);
+  if (typeof value === 'string') {
+    var intArray = intArrayFromString(value);
+    var offset = ensureCache.alloc(intArray, HEAP8);
+    ensureCache.copy(intArray, HEAP8, offset);
+    return offset;
+  }
+  return value;
+}
+function ensureInt8(value) {
+  if (typeof value === 'object') {
+    var offset = ensureCache.alloc(value, HEAP8);
+    ensureCache.copy(value, HEAP8, offset);
+    return offset;
+  }
+  return value;
+}
+function ensureInt16(value) {
+  if (typeof value === 'object') {
+    var offset = ensureCache.alloc(value, HEAP16);
+    ensureCache.copy(value, HEAP16, offset);
+    return offset;
+  }
+  return value;
+}
+function ensureInt32(value) {
+  if (typeof value === 'object') {
+    var offset = ensureCache.alloc(value, HEAP32);
+    ensureCache.copy(value, HEAP32, offset);
+    return offset;
+  }
+  return value;
+}
+function ensureFloat32(value) {
+  if (typeof value === 'object') {
+    var offset = ensureCache.alloc(value, HEAPF32);
+    ensureCache.copy(value, HEAPF32, offset);
+    return offset;
+  }
+  return value;
+}
+function ensureFloat64(value) {
+  if (typeof value === 'object') {
+    var offset = ensureCache.alloc(value, HEAPF64);
+    ensureCache.copy(value, HEAPF64, offset);
+    return offset;
+  }
   return value;
 }
 
@@ -200,7 +260,7 @@ mid_c += ['''
 // Not using size_t for array indices as the values used by the javascript code are signed.
 void array_bounds_check(const int array_size, const int array_idx) {
   if (array_idx < 0 || array_idx >= array_size) {
-    EM_ASM_INT({
+    EM_ASM({
       throw 'Array index ' + $0 + ' out of bounds: [0,' + $1 + ')';
     }, array_idx, array_size);
   }
@@ -209,37 +269,46 @@ void array_bounds_check(const int array_size, const int array_idx) {
 
 C_FLOATS = ['float', 'double']
 
+def full_typename(arg):
+  return arg.type.name + ('[]' if arg.type.isArray() else '')
+
 def type_to_c(t, non_pointing=False):
   #print 'to c ', t
+  def base_type_to_c(t):
+    if t == 'Long':
+      ret = 'int'
+    elif t == 'UnsignedLong':
+      ret = 'unsigned int'
+    elif t == 'Short':
+      ret = 'short'
+    elif t == 'UnsignedShort':
+      ret = 'unsigned short'
+    elif t == 'Byte':
+      ret = 'char'
+    elif t == 'Octet':
+      ret = 'unsigned char'
+    elif t == 'Void':
+      ret = 'void'
+    elif t == 'String':
+      ret = 'char*'
+    elif t == 'Float':
+      ret = 'float'
+    elif t == 'Double':
+      ret = 'double'
+    elif t == 'Boolean':
+      ret = 'bool'
+    elif t == 'Any' or t == 'VoidPtr':
+      ret = 'void*'
+    elif t in interfaces:
+      ret = (interfaces[t].getExtendedAttribute('Prefix') or [''])[0] + t + ('' if non_pointing else '*')
+    else:
+      ret = t
+    return ret
+
   t = t.replace(' (Wrapper)', '')
-  if t == 'Long':
-    return 'int'
-  elif t == 'UnsignedLong':
-    return 'unsigned int'
-  elif t == 'Short':
-    return 'short'
-  elif t == 'UnsignedShort':
-    return 'unsigned short'
-  elif t == 'Byte':
-    return 'char'
-  elif t == 'Octet':
-    return 'unsigned char'
-  elif t == 'Void':
-    return 'void'
-  elif t == 'String':
-    return 'char*'
-  elif t == 'Float':
-    return 'float'
-  elif t == 'Double':
-    return 'double'
-  elif t == 'Boolean':
-    return 'bool'
-  elif t == 'Any' or t == 'VoidPtr':
-    return 'void*'
-  elif t in interfaces:
-    return (interfaces[t].getExtendedAttribute('Prefix') or [''])[0] + t + ('' if non_pointing else '*')
-  else:
-    return t
+  if '[]' in t:
+    return base_type_to_c(t.replace('[]', '')) + '*'
+  return base_type_to_c(t)
 
 def take_addr_if_nonpointer(m):
   if m.getExtendedAttribute('Ref') or m.getExtendedAttribute('Value'):
@@ -261,7 +330,7 @@ def type_to_cdec(raw):
     return ret
   return ret + '*'
 
-def render_function(class_name, func_name, sigs, return_type, non_pointer, copy, operator, constructor, func_scope, call_content=None, const=False):
+def render_function(class_name, func_name, sigs, return_type, non_pointer, copy, operator, constructor, func_scope, call_content=None, const=False, array_attribute=False):
   global mid_c, mid_js, js_impl_methods
 
   legacy_mode = CHECKS not in ['ALL', 'FAST']
@@ -271,17 +340,16 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
   min_args = min(sigs.keys())
   max_args = max(sigs.keys())
 
-  c_names = {}
   all_args = sigs.get(max_args)
 
   if DEBUG:
-    print 'renderfunc', class_name, func_name, sigs.keys(), return_type, constructor
+    print('renderfunc', class_name, func_name, list(sigs.keys()), return_type, constructor)
     for i in range(max_args):
       a = all_args[i]
       if isinstance(a, WebIDL.IDLArgument):
-        print ("  arg%d" % i), a.identifier, a.type, a.optional
+        print(("  arg%d" % i), a.identifier, a.type, a.optional)
       else:
-        print "  arg%d" % i
+        print("  arg%d" % i)
 
   # JS
 
@@ -308,20 +376,16 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
     body = ''
     pre_arg = []
 
-  for i in range(max_args):
-    arg = all_args[i]
-    if arg.type.isString():
-      body += '  ensureStringCache.prepare();\n'
-      break
+  if any(arg.type.isString() or arg.type.isArray() for arg in all_args):
+    body += '  ensureCache.prepare();\n'
 
   full_name = "%s::%s" % (class_name, func_name)
 
-  for i in range(max_args):
+  for i, (js_arg, arg) in enumerate(zip(args, all_args)):
     if i >= min_args:
       optional = True
     else:
       optional = False
-    arg = all_args[i]
     do_default = False
     # Filter out arguments we don't know how to parse. Fast casing only common cases.
     compatible_arg = isinstance(arg, Dummy) or (isinstance(arg, WebIDL.IDLArgument) and arg.optional is False)
@@ -332,38 +396,38 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
       else:
         arg_name = ''
       # Format assert fail message
-      check_msg = "[CHECK FAILED] %s(arg%d:%s): " % (full_name, i, arg_name)
+      check_msg = "[CHECK FAILED] %s(%s:%s): " % (full_name, js_arg, arg_name)
       if isinstance(arg.type, WebIDL.IDLWrapperType):
         inner = arg.type.inner
       else:
         inner = ""
 
       # Print type info in comments.
-      body += "  /* arg%d <%s> [%s] */\n" % (i, arg.type.name, inner)
+      body += "  /* %s <%s> [%s] */\n" % (js_arg, arg.type.name, inner)
 
-      # Wrap asserts with existance check when argument is optional.
-      if all_checks and optional: body += "if(typeof arg%d !== 'undefined' && arg%d !== null) {\n" % (i, i)
+      # Wrap asserts with existence check when argument is optional.
+      if all_checks and optional: body += "if(typeof {0} !== 'undefined' && {0} !== null) {{\n".format(js_arg)
       # Special case argument types.
       if arg.type.isNumeric():
         if arg.type.isInteger():
-          if all_checks: body += "  assert(typeof arg%d === 'number' && !isNaN(arg%d), '%sExpecting <integer>');\n" % (i, i, check_msg)
+          if all_checks: body += "  assert(typeof {0} === 'number' && !isNaN({0}), '{1}Expecting <integer>');\n".format(js_arg, check_msg)
         else:
-          if all_checks: body += "  assert(typeof arg%d === 'number', '%sExpecting <number>');\n" % (i, check_msg)
+          if all_checks: body += "  assert(typeof {0} === 'number', '{1}Expecting <number>');\n".format(js_arg, check_msg)
         # No transform needed for numbers
       elif arg.type.isBoolean():
-        if all_checks: body += "  assert(typeof arg%d === 'boolean' || (typeof arg%d === 'number' && !isNaN(arg%d)), '%sExpecting <boolean>');\n" % (i, i, i, check_msg)
+        if all_checks: body += "  assert(typeof {0} === 'boolean' || (typeof {0} === 'number' && !isNaN({0})), '{1}Expecting <boolean>');\n".format(js_arg, check_msg)
         # No transform needed for booleans
       elif arg.type.isString():
         # Strings can be DOM strings or pointers.
-        if all_checks: body += "  assert(typeof arg%d === 'string' || (arg%d && typeof arg%d === 'object' && typeof arg%d.ptr === 'number'), '%sExpecting <string>');\n" % (i, i, i, i, check_msg)
+        if all_checks: body += "  assert(typeof {0} === 'string' || ({0} && typeof {0} === 'object' && typeof {0}.ptr === 'number'), '{1}Expecting <string>');\n".format(js_arg, check_msg)
         do_default = True # legacy path is fast enough for strings.
       elif arg.type.isInterface():
-        if all_checks: body += "  assert(typeof arg%d === 'object' && typeof arg%d.ptr === 'number', '%sExpecting <pointer>');\n" % (i, i, check_msg)
+        if all_checks: body += "  assert(typeof {0} === 'object' && typeof {0}.ptr === 'number', '{1}Expecting <pointer>');\n".format(js_arg, check_msg)
         if optional:
-          body += "  if(typeof arg%d !== 'undefined' && arg%d !== null) { arg%d = arg%d.ptr };\n" % (i, i, i, i)
+          body += "  if(typeof {0} !== 'undefined' && {0} !== null) {{ {0} = {0}.ptr }};\n".format(js_arg)
         else:
           # No checks in fast mode when the arg is required
-          body += "  arg%d = arg%d.ptr;\n" % (i, i)
+          body += "  {0} = {0}.ptr;\n".format(js_arg)
       else:
         do_default = True
 
@@ -372,19 +436,33 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
       do_default = True
 
     if do_default:
-      body += "  if (arg%d && typeof arg%d === 'object') arg%d = arg%d.ptr;\n" % (i, i, i, i)
-      if arg.type.isString():
-        body += "  else arg%d = ensureString(arg%d);\n" % (i, i)
-
+      if not (arg.type.isArray() and not array_attribute):
+        body += "  if ({0} && typeof {0} === 'object') {0} = {0}.ptr;\n".format(js_arg)
+        if arg.type.isString():
+          body += "  else {0} = ensureString({0});\n".format(js_arg)
+      else:
+        # an array can be received here
+        arg_type = arg.type.name
+        if arg_type in ['Byte', 'Octet']:
+          body += "  if (typeof {0} == 'object') {{ {0} = ensureInt8({0}); }}\n".format(js_arg)
+        elif arg_type in ['Short', 'UnsignedShort']:
+          body += "  if (typeof {0} == 'object') {{ {0} = ensureInt16({0}); }}\n".format(js_arg)
+        elif arg_type in ['Long', 'UnsignedLong']:
+          body += "  if (typeof {0} == 'object') {{ {0} = ensureInt32({0}); }}\n".format(js_arg)
+        elif arg_type == 'Float':
+          body += "  if (typeof {0} == 'object') {{ {0} = ensureFloat32({0}); }}\n".format(js_arg)
+        elif arg_type == 'Double':
+          body += "  if (typeof {0} == 'object') {{ {0} = ensureFloat64({0}); }}\n".format(js_arg)
 
+  c_names = {}
   for i in range(min_args, max_args):
     c_names[i] = 'emscripten_bind_%s_%d' % (bindings_name, i)
-    body += '  if (arg%d === undefined) { %s%s(%s)%s%s }\n' % (i, call_prefix, '_' + c_names[i], ', '.join(pre_arg + args[:i]), call_postfix, '' if 'return ' in call_prefix else '; ' + (cache or ' ') + 'return')
+    body += '  if (%s === undefined) { %s%s(%s)%s%s }\n' % (args[i], call_prefix, '_' + c_names[i], ', '.join(pre_arg + args[:i]), call_postfix, '' if 'return ' in call_prefix else '; ' + (cache or ' ') + 'return')
   c_names[max_args] = 'emscripten_bind_%s_%d' % (bindings_name, max_args)
   body += '  %s%s(%s)%s;\n' % (call_prefix, '_' + c_names[max_args], ', '.join(pre_arg + args), call_postfix)
   if cache:
     body += '  ' + cache + '\n'
-  mid_js += [r'''function%s(%s) {
+  mid_js += [r'''/** @suppress {undefinedVars, duplicate} */function%s(%s) {
 %s
 };''' % ((' ' + func_name) if constructor else '', ', '.join(args), body[:-1])]
 
@@ -393,16 +471,18 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
   for i in range(min_args, max_args+1):
     raw = sigs.get(i)
     if raw is None: continue
-    sig = [arg.type.name for arg in raw]
+    sig = list(map(full_typename, raw))
+    if array_attribute:
+      sig = [x.replace('[]', '') for x in sig] # for arrays, ignore that this is an array - our get/set methods operate on the elements
 
-    c_arg_types = map(type_to_c, sig)
+    c_arg_types = list(map(type_to_c, sig))
 
-    normal_args = ', '.join(['%s arg%d' % (c_arg_types[j], j) for j in range(i)])
+    normal_args = ', '.join(['%s %s' % (c_arg_types[j], args[j]) for j in range(i)])
     if constructor:
       full_args = normal_args
     else:
       full_args = type_to_c(class_name, non_pointing=True) + '* self' + ('' if not normal_args else ', ' + normal_args)
-    call_args = ', '.join(['%sarg%d' % ('*' if raw[j].getExtendedAttribute('Ref') else '', j) for j in range(i)])
+    call_args = ', '.join(['%s%s' % ('*' if raw[j].getExtendedAttribute('Ref') else '', args[j]) for j in range(i)])
     if constructor:
       call = 'new ' + type_to_c(class_name, non_pointing=True)
       call += '(' + call_args + ')'
@@ -413,12 +493,17 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
       call += '(' + call_args + ')'
 
     if operator:
-      assert '=' in operator, 'can only do += *= etc. for now, all with "="'
       cast_self = 'self'
       if class_name != func_scope:
         # this function comes from an ancestor class; for operators, we must cast it
         cast_self = 'dynamic_cast<' + type_to_c(func_scope) + '>(' + cast_self + ')'
-      call = '(*%s %s %sarg0)' % (cast_self, operator, '*' if sig[0] in interfaces else '')
+      maybe_deref = '*' if sig[0] in interfaces else ''
+      if '=' in operator:
+        call = '(*%s %s %s%s)' % (cast_self, operator, maybe_deref, args[0])
+      elif operator == '[]':
+        call = '((*%s)[%s%s])' % (cast_self, maybe_deref, args[0])
+      else:
+        raise Exception('unfamiliar operator ' + operator)
 
     pre = ''
 
@@ -441,8 +526,8 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
 
     if not constructor:
       if i == max_args:
-        dec_args = ', '.join(map(lambda j: type_to_cdec(raw[j]) + ' arg' + str(j), range(i)))
-        js_call_args = ', '.join(['%sarg%d' % (('(int)' if sig[j] in interfaces else '') + ('&' if raw[j].getExtendedAttribute('Ref') or raw[j].getExtendedAttribute('Value') else ''), j) for j in range(i)])
+        dec_args = ', '.join([type_to_cdec(raw[j]) + ' ' + args[j] for j in range(i)])
+        js_call_args = ', '.join(['%s%s' % (('(int)' if sig[j] in interfaces else '') + take_addr_if_nonpointer(raw[j]), args[j]) for j in range(i)])
 
         js_impl_methods += [r'''  %s %s(%s) {
     %sEM_ASM_%s({
@@ -461,7 +546,7 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
           (', ' if js_call_args else '') + js_call_args)]
 
 
-for name, interface in interfaces.iteritems():
+for name, interface in interfaces.items():
   js_impl = interface.getExtendedAttribute('JSImplementation')
   if not js_impl: continue
   implements[name] = [js_impl[0]]
@@ -475,7 +560,7 @@ for name, interface in interfaces.iteritems():
 # that invariant. Further, the height of a node never decreases. Therefore, when the loop
 # finishes, all ancestors of a given node should have a larger height number than that node.
 nodeHeight = {}
-for child, parent in implements.iteritems():
+for child, parent in implements.items():
   parent = parent[0]
   while parent:
     nodeHeight[parent] = max(nodeHeight.get(parent, 0), nodeHeight.get(child, 0) + 1)
@@ -486,8 +571,7 @@ for child, parent in implements.iteritems():
     else:
       parent = None
 
-names = interfaces.keys()
-names.sort(lambda x, y: nodeHeight.get(y, 0) - nodeHeight.get(x, 0))
+names = sorted(interfaces.keys(), key=lambda x: nodeHeight.get(x, 0), reverse=True)
 
 for name in names:
   interface = interfaces[name]
@@ -507,14 +591,10 @@ for name in names:
 
   # Methods
 
-  seen_constructor = False # ensure a constructor, even for abstract base classes
-  for m in interface.members:
-    if m.identifier.name == name:
-      seen_constructor = True
-      break
-  if not seen_constructor:
+  # Ensure a constructor even if one is not specified.
+  if not any(m.identifier.name == name for m in interface.members):
     mid_js += ['function %s() { throw "cannot construct a %s, no constructor in IDL" }\n' % (name, name)]
-    emit_constructor(name)
+    mid_js += build_constructor(name)
 
   for m in interface.members:
     if not m.isMethod(): continue
@@ -552,7 +632,7 @@ for name in names:
                     const=m.getExtendedAttribute('Const'))
     mid_js += [';\n']
     if constructor:
-      emit_constructor(name)
+      mid_js += build_constructor(name)
 
   for m in interface.members:
     if not m.isAttr(): continue
@@ -585,7 +665,8 @@ for name in names:
                     False,
                     func_scope=interface,
                     call_content=get_call_content,
-                    const=m.getExtendedAttribute('Const'))
+                    const=m.getExtendedAttribute('Const'),
+                    array_attribute=m.type.isArray())
 
     if not m.readonly:
       set_name = 'set_' + attr
@@ -599,7 +680,8 @@ for name in names:
                       False,
                       func_scope=interface,
                       call_content=set_call_content,
-                      const=m.getExtendedAttribute('Const'))
+                      const=m.getExtendedAttribute('Const'),
+                      array_attribute=m.type.isArray())
 
   if not interface.getExtendedAttribute('NoDelete'):
     mid_js += [r'''
@@ -625,28 +707,29 @@ public:
 
 deferred_js = []
 
-for name, enum in enums.iteritems():
+for name, enum in enums.items():
   mid_c += ['\n// ' + name + '\n']
   deferred_js += ['\n', '// ' + name + '\n']
   for value in enum.values():
     function_id = "%s_%s" % (name, value.split('::')[-1])
-    mid_c += [r'''%s EMSCRIPTEN_KEEPALIVE emscripten_enum_%s() {
+    function_id = 'emscripten_enum_%s' % function_id
+    mid_c += [r'''%s EMSCRIPTEN_KEEPALIVE %s() {
   return %s;
 }
 ''' % (name, function_id, value)]
     symbols = value.split('::')
     if len(symbols) == 1:
       identifier = symbols[0]
-      deferred_js += ["Module['%s'] = _emscripten_enum_%s();\n" % (identifier, function_id)]
+      deferred_js += ["Module['%s'] = _%s();\n" % (identifier, function_id)]
     elif len(symbols) == 2:
       [namespace, identifier] = symbols
       if namespace in interfaces:
         # namespace is a class
-        deferred_js += ["Module['%s']['%s'] = _emscripten_enum_%s();\n" % \
+        deferred_js += ["Module['%s']['%s'] = _%s();\n" % \
                   (namespace, identifier, function_id)]
       else:
         # namespace is a namespace, so the enums get collapsed into the top level namespace.
-        deferred_js += ["Module['%s'] = _emscripten_enum_%s();\n" % (identifier, function_id)]
+        deferred_js += ["Module['%s'] = _%s();\n" % (identifier, function_id)]
     else:
       raise Exception("Illegal enum value %s" % value)
 
@@ -663,12 +746,10 @@ mid_js += ['''
 
 # Write
 
-c = open(output_base + '.cpp', 'w')
-for x in pre_c: c.write(x)
-for x in mid_c: c.write(x)
-c.close()
+with open(output_base + '.cpp', 'w') as c:
+  for x in pre_c: c.write(x)
+  for x in mid_c: c.write(x)
 
-js = open(output_base + '.js', 'w')
-for x in mid_js: js.write(x)
-js.close()
+with open(output_base + '.js', 'w') as js:
+  for x in mid_js: js.write(x)
 
